; TIME_DIV.ASM
;
; External assembler subroutine for Recwav.pas.  This routine measures the
; amount of time in PIT clock counts that it takes to fill a 32768-byte buffer
; with sound, using a given recording divider, and returns the count.  This
; routine assumes the DAC has already been initialized.
;
; The Turbo Pascal invocation syntax is:
;
; function time_div(
;   buffer:                    (* buffer to fill *)
;     pointer;
;   divider:                   (* divider to use *)
;     word ):
;       longint;
;

CODE		SEGMENT	BYTE PUBLIC
		ASSUME	CS:CODE,DS:CODE
		PUBLIC	TIME_DIV
TIME_DIV	PROC	NEAR
		JMP	START
		;
		; Stack frame structure.
		;
STACKFRAME	STRUC	[BP]
OLDBP		DW	?	; caller's BP
RETADDR		DW	?	; return address
DIVIDER		DW	?	; divider for Int 1Ah function 82h
BUFFER		DD      ?       ; pointer to DMA buffer
		ENDS
		;
		; Local data.
		;
SOUNDHALTED	DB	0	; flag, 1 when Int 1Ah function complete
INT15OLD	DD	0	; default vector for Int 15h
STARTCOUNT	DD	0	; starting count
		;
		; Temporary Int 15h handler.
		;
INT15NEW:	CMP	AX,91FBh
		JE	>L0
		JMP	DWORD PTR CS:INT15OLD
L0:		MOV	CS:SOUNDHALTED,1
		IRET
		;
		; Subroutine, returns the current timer clock count in DX:AX.
		; Requires an 8254; also requires that the timer not have been
		; "speeded up" by some TSR.
		;
GETTIME:	PUSH	BX
		PUSH	CX
		PUSH	SI
		PUSH	ES
		MOV	AX,40h		; ES addresses BIOS data segment
		MOV	ES,AX
		;
		; Get time.  The count must be safely away from where timer
		; ticks occur, or where the OUT pin for channel 0 changes,
		; or the time returned could be wrong.
		;
GETTIME_LOOP:	CLI
		MOV	AL,0C2h		; 8254 read back command
		OUT	43h,AL
		JMP	$+2
		IN	AL,40h		; read channel 0 status
		JMP	$+2
		MOV	BH,AL
		IN	AL,40h		; read channel 0 count (low byte)
		JMP	$+2
		MOV	CL,AL
		IN	AL,40h		; read channel 0 count (high byte)
		JMP	$+2
		MOV	CH,AL		; count in CX now
		MOV	SI,ES:[6Ch]	; get low word of BIOS clock count
		STI
		MOV	AX,CX		; AX := |count|
		CWD
		XOR	AX,DX
		SUB	AX,DX
		CMP	AX,1024		; if |count| < 1024, read time again
		JB	GETTIME_LOOP
		;
		; Adjust count to give count 0-65535, return in AX.
		;
		SHR	CX,1
		AND	BH,80h
		ADD	CH,BH
		MOV	AX,CX
		NEG	AX
		;
		; Return low BIOS clock word in DX.
		;
		MOV	DX,SI
		;
		; Restore registers and return.
		;
		POP	ES
		POP	SI
		POP	CX
		POP	BX
		RET
		;
		; Main routine.  BP addresses the stack frame; DS addresses
		; the code segment.
		;
START:		PUSH	BP
		MOV	BP,SP
		PUSH	DS
		PUSH	CS
		POP	DS
		;
		; Save the default Int 15h vector.
		;
		MOV	AX,3515h
		INT	21h
		MOV	WORD PTR INT15OLD,BX
		MOV	WORD PTR INT15OLD+2,ES
		;
		; Use the new Int 15h handler now.
		;
		MOV	DX,OFFSET INT15NEW
		MOV	AX,2515h
		INT	21h
		;
		; Get starting count.
		;
		CALL	GETTIME
		MOV	WORD PTR STARTCOUNT,AX
		MOV	WORD PTR STARTCOUNT+2,DX
		;
		; Start recording.
		;
		MOV	SOUNDHALTED,0
		MOV	AH,82h
		LES	BX,BUFFER
		MOV	CX,32768
		MOV	DX,DIVIDER
		INT	1Ah
		;
		; Wait until done.
		;
WAITLOOP:	CMP	SOUNDHALTED,1
		JNE	WAITLOOP
		;
		; Get stopping count, subtract starting count, and save
		; elapsed time on stack.
		;
		CALL	GETTIME
		SUB	AX,WORD PTR STARTCOUNT
		SBB	DX,WORD PTR STARTCOUNT+2
		PUSH	AX
		PUSH	DX
		;
		; Restore the default Int 15h vector.
		;
		LDS	DX,INT15OLD
		MOV	AX,2515h
		INT	21h
		;
		; Get back elapsed time, restore caller's DS and BP, and
		; return.
		;
		POP	DX
		POP	AX
		POP	DS
		POP	BP
		RET	6	; discard parameters
TIME_DIV	ENDP
CODE		ENDS
		END
